package ru.dbotthepony.kstarbound.lua;

import jnr.ffi.LibraryLoader;
import jnr.ffi.Pointer;
import org.jetbrains.annotations.NotNull;

@SuppressWarnings({"UnnecessaryModifier", "SpellCheckingInspection", "unused"})
public interface LuaJNR {
	public int lua_pcallk(long luaState, int numArgs, int numResults, int msgh, long ctx, LuaJNA.lua_KFunction callback);
	public int lua_callk(long luaState, int numArgs, int numResults, long ctx, LuaJNA.lua_KFunction callback);
	public long lua_atpanic(long luaState, @NotNull LuaJNA.lua_CFunction fn);

	public long luaL_newstate();
	public void lua_close(long luaState);

	// Стандартные библиотеки
	public void luaopen_base(long luaState);
	public void luaopen_package(long luaState);
	public void luaopen_coroutine(long luaState);
	public void luaopen_table(long luaState);
	public void luaopen_io(long luaState);
	public void luaopen_os(long luaState);
	public void luaopen_string(long luaState);
	public void luaopen_math(long luaState);
	public void luaopen_utf8(long luaState);
	public void luaopen_debug(long luaState);

	public long lua_tolstring(long luaState, int index, @NotNull Pointer size);

	public int lua_load(long luaState, @NotNull lua_Reader reader, long userData, @NotNull String chunkName, @NotNull String mode);

	public interface lua_CFunction {
		int invoke(long luaState);
	}

	public interface lua_Reader {
		public long readNextChunk(long luaState, long userData, @NotNull Pointer sizeToRead);
	}

	public interface lua_KFunction {
		public int invoke(long luaState, int status, long context);
	}

	// Операции над стаком
	// загрузка значений из Java на стек
	public void lua_createtable(long luaState, int arraySize, int hashSize);
	public void lua_pushnil(long luaState);
	public void lua_pushnumber(long luaState, double value);
	public void lua_pushinteger(long luaState, long value);
	public void lua_pushboolean(long luaState, int value);

	// NUL терминированная строка
	public void lua_pushstring(long luaState, @NotNull String value);

	// двоичная строка
	public long lua_pushlstring(long luaState, long stringPointer, long length);

	// Загрузка Lua значений на стек
	public int lua_getglobal(long luaState, @NotNull String name);

	// запись значений со стека
	public void lua_settable(long luaState, int stackIndex);
	public void lua_setglobal(long luaState, @NotNull String name);

	public int lua_checkstack(long luaState, int value);
	public int lua_absindex(long luaState, int value);

	/**
	 * Returns the index of the top element in the stack.
	 * Because indices start at 1, this result is equal to the number of elements in the stack; in particular, 0 means an empty stack.
	 */
	public int lua_gettop(long luaState);

	public static final LuaJNR INSTANCE = LibraryLoader.create(LuaJNR.class).load("lua54");
}
